<?php

namespace EscapeWork\Manager\Providers;

use EscapeWork\Manager\Permission;
use EscapeWork\Manager\Validators\PasswordMatchValidator;
use Illuminate\Support\ServiceProvider;
use Illuminate\Foundation\AliasLoader;
use Auth;

class ManagerServiceProvider extends ServiceProvider
{

    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        /**
         * Indicates package root folder
         */
        $root = __DIR__ . '/../..';

        # publiishing files
        $this->publishes([
            $root . '/resources/assets/css'                                     => public_path('packages/manager/css'),
            $root . '/resources/assets/img'                                     => public_path('packages/manager/img'),
            $root . '/resources/assets/js'                                      => public_path('packages/manager/js'),
            $root . '/resources/assets/vendor/bootstrap'                        => public_path('packages/manager/vendor/bootstrap'),
            $root . '/resources/assets/vendor/all.min.js'                       => public_path('packages/manager/vendor/all.min.js'),
            $root . '/resources/assets/vendor/medias.min.js'                    => public_path('packages/manager/vendor/medias.min.js'),
            $root . '/resources/assets/vendor/medium-editor/dist'               => public_path('packages/manager/vendor/medium-editor/dist'),
            $root . '/resources/assets/vendor/medium-editor-insert-plugin/dist' => public_path('packages/manager/vendor/medium-editor-insert-plugin/dist'),
            $root . '/resources/assets/vendor/font-awesome/css'                 => public_path('packages/manager/vendor/font-awesome/css'),
            $root . '/resources/assets/vendor/font-awesome/fonts'               => public_path('packages/manager/vendor/font-awesome/fonts'),
        ], 'public');

        $this->publishes([
            $root . '/config/manager.php'  => config_path('manager.php'),
            $root . '/database/migrations' => 'database/migrations',
        ], 'config');

        # loading views and translations from resources
        $this->loadViewsFrom($root . '/resources/views', 'manager');
        $this->loadTranslationsFrom($root . '/resources/translations', 'manager');
        $this->loadValidators();

        $this->createConfigurations();
    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->registerMiddlewares();
        $this->registerPermission();

        $this->mergeConfigFrom(
            __DIR__ . '/../../config/manager.php', 'manager'
        );

        $this->callProviders();
        $this->registerCommands();
    }

    protected function registerMiddlewares()
    {
        $kernel = new \EscapeWork\Manager\Http\Kernel;

        foreach ($kernel->getMiddlewares() as $key => $middleware) {
            $this->app->router->middleware($key, $middleware);
        }
    }

    /**
     * Registering the permission check class
     */
    protected function registerPermission()
    {
        $this->app['escapework.manager.permission'] = $this->app->share(function($app)
        {
            $user       = Auth::guard('manager')->user();
            $permission = $app->make('EscapeWork\Manager\Permission', [$user]);

            return $permission;
        });
    }

    /**
     * Calling the providers needed
     */
    protected function callProviders()
    {
        $this->app->register('EscapeWork\Manager\Providers\RouteServiceProvider');
        $this->app->register('EscapeWork\Manager\Providers\UserServiceProvider');
        $this->app->register('EscapeWork\Manager\Providers\ResponseServiceProvider');

        if ($this->app['request']->is(config('manager.url') . '/*')) {
            $this->app->register('Rap2hpoutre\LaravelLogViewer\LaravelLogViewerServiceProvider');

            # handling manager errors
            $this->app->singleton(
                'Illuminate\Contracts\Debug\ExceptionHandler',
                'EscapeWork\Manager\Exceptions\Handler'
            );
        }

        # se for em ambiente local, adiciona o generator service provider do jeffrey
        if ($this->app->environment() == 'local') {
            $this->app->register('Laracasts\Generators\GeneratorsServiceProvider');
        }
    }

    /**
     * Registrando os comandos
     */
    public function registerCommands()
    {
        $this->commands(
            'EscapeWork\Manager\Console\ConfigureCommand',
            'EscapeWork\Manager\Console\CreateUserCommand',
            'EscapeWork\Manager\Console\UpdatePasswordCommand',

            'EscapeWork\Manager\Console\CrudGeneratorCommand',
            'EscapeWork\Manager\Console\CrudGeneratorTemplateCommand',
            'EscapeWork\Manager\Console\ControllerMakeCommand',
            'EscapeWork\Manager\Console\RequestMakeCommand',
            'EscapeWork\Manager\Console\ModelMakeCommand',
            'EscapeWork\Manager\Console\RepositoryMakeCommand',
            'EscapeWork\Manager\Console\ViewsMakeCommand',
            'EscapeWork\Manager\Console\PresenterMakeCommand',
            'EscapeWork\Manager\Console\MigrationMakeCommand'
        );
    }

    /**
     * Criando as configurações em tempo de execução
     */
    protected function createConfigurations()
    {
        $this->app['config']->set('manager.packages', []);

        $this->app['config']['manager.permissions.pages'] = array_merge(
            $this->app['config']['manager.permissions.pages'],
            [
                'users' => [
                    'title'   => trans('manager::users.title'),
                    'actions' => [], // ex: create, read, update, delete, etc
                ],

                'roles' => [
                    'title'   => trans('manager::roles.title'),
                    'actions' => [], // ex: create, read, update, delete, etc
                ],
            ]
        );
    }

    /**
     * Resolving validators
     * @return void
     */
    protected function loadValidators()
    {
        $this->app->validator->extend('password_match', 'EscapeWork\Manager\Validators\PasswordMatchValidator@validate');
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return [
            'escapework.manager.permission',
        ];
    }
}
